/***************************************************************************
                           bombs.c
    description          : bomb handling
                           -------------------
    begin                : Fri March 30 2001, from rungame.c
    date                 : 01/08/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <string.h>  //memset()
#include "data.h"    //LoadImage()
#include "structs.h" //global screen
#include "bombs.h"

#define B_TIMER     20
#define MAX_BOMBS   30

static Sprite bombs[MAX_BOMBS];
static int bomb_max_y;
static int bomb_countdown;

static int bomb_count;
static int bomb_speed;


/**
 *  bombs_init()
 *  initializes bomb data
 *
 *  Returns:    -1 on error, otherwise count of Sprites.
 *
**/
int bombs_init(void)
{
	int i;

	memset(&bombs, 0, MAX_BOMBS * sizeof(Sprite));
	if ((bombs[0].image = LoadImage(DATAFILE("bomb.gif"), 0)) == NULL)
		return -1;
	for (i = 1; i < MAX_BOMBS; ++i)
		bombs[i].image = bombs[0].image;
	bomb_max_y = screen->h - bombs[0].image->h;
	bomb_countdown = B_TIMER;
	return MAX_BOMBS;
}


/**
 *  bombs_free()
 *  frees bomb data
 *
**/
void bombs_free(void)
{
	if (bombs[0].image)
		SDL_FreeSurface(bombs[0].image);
}


/**
 *  bombs_set_count()
 *  set maximum number of bombs, clamped 2 to MAX_BOMBS.
 *
 *  @count:      allowed number of bombs.
 *
**/
void bombs_set_count(int count)
{
	if (count < 2)
		count = 2;
	if (count > MAX_BOMBS)  //Dont exceed allocation;
		count = MAX_BOMBS;
	bomb_count = count;
}


/**
 *  bombs_set_speed()
 *  set pixels bomb moves, clamped 1 to 20.
 *
 *  @speed:      allowed speed.
 *
**/
void bombs_set_speed(int speed)
{
	if (speed < 1)
		speed = 1;
	if (speed > 20)  //Should be fast enough for even 1200 screen height;
		speed = 20;
	bomb_speed = speed;
}


/**
 *  bombs_add()
 *  find a free bomb slot, if it is time
 *
 *  Returns:    pointer to a free bomb Sprite, or NULL.
 *
**/
Sprite *bombs_add(void)
{
	int i;

	if (bomb_countdown > 0) //Time to drop a bomb?
		--bomb_countdown;   //No
	else                    //Yes
	{
		for (i = bomb_count - 1; i >= 0; --i)
		{
			if (!bombs[i].alive)
			{
				//reset bomb "timer"
				bomb_countdown = B_TIMER;
				return &bombs[i];
			}
		}
	}
	return NULL;
}


/**
 *  bombs_move()
 *  move all bombs down, kill if off bottom
 *
**/
void bombs_move(void)
{
	int i;

	for (i = bomb_count - 1; i >= 0; --i)
	{
		if ( bombs[i].alive )
		{
			bombs[i].y += bomb_speed;
			if (bombs[i].y > bomb_max_y)
				bombs[i].alive = 0;
		}
	}
}


/**
 *  bombs_collide()
 *  check all bombs for collision with sprite, kill bomb if collided.
 *
 *  @s:         Sprite to test.
 *
 *  Returns:    1 on collision, else 0.
 *
**/
int bombs_collide(Sprite *s)
{
	int i;
	int hit = 0;

	for (i = bomb_count -1; i >= 0; --i)
	{
		if (bombs[i].alive && sprite_collide(s, &bombs[i]))
		{
			bombs[i].alive = 0;
			hit = 1;
		}
	}
	return hit;
}


/**
 *  bombs_foreach()
 *  apply function to all bombs.
 *
 *  @dofunc:    function to apply.
 *
**/
void bombs_foreach(SpriteFunc dofunc)
{
	int i;

	for (i = bomb_count - 1; i >= 0; --i)
	{
		if (bombs[i].alive)
			dofunc(&bombs[i]);
	}
}
