/***************************************************************************
                          final_score.c
    description:          show final score
                             -------------------
    begin                : Fri March 30 2001
    date                 : 27/07/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "structs.h"
#include "sprite.h"
#include "data.h"
#include "hi_score.h"
#include "final_score.h"

/* Final Score. */
#define FS_X            400     // x offset of score
#define FS_FONT_W       40      // 'font' width

SDL_Surface *initials;
SDL_Surface *finalscore;

int final_score_init(void)
{
	if ((initials = LoadImage(DATAFILE("initials.gif"), 0)) == NULL)
		return(0);
	if ((finalscore = LoadImage(DATAFILE("finalscore.gif"), 0)) == NULL)
		return(0);
	return 1;
}


void final_score_free(void)
{
	free(initials);
	free(finalscore);
}


static void _final_score_draw(int score, int position)
{
	static SDL_Rect srect = {0, 0, 30, 30};
	SDL_Rect drect;
	int i, digits;
	char score_int[MAX_DIGITS + 2];

	//Paint the background
	drect.x = 0;
	drect.y = 0;
	SDL_BlitSurface(background, NULL, screen, &drect);

	//Paint the message
	drect.x = 72;
	drect.y = 150;
	SDL_BlitSurface(finalscore, NULL, screen, &drect);

	//Paint the prompt
	if (position < HS_MAX_SCORES)
	{
		drect.x = 56;
		drect.y = 208;
		SDL_BlitSurface(initials, NULL, screen, &drect);
	}

	//Paint the score
	if (score > SCORE_MAX || score < 0) // sanity check
		score = 0;
	digits = snprintf(score_int, MAX_DIGITS + 1, "%d", score);
	for (i = 0; i < digits; ++i)
	{
		drect.x = FS_X + (FS_FONT_W * i);
		drect.y = 155;
		srect.x = (score_int[i] - '0') * 30;
		SDL_BlitSurface(digit_30_image, &srect, screen, &drect);
	}
	SDL_UpdateRect(screen, 0, 0, 0, 0);
}


int final_score_view(int score)
{
	char *buf;
	SDL_Event event;
	char score_buf[MAX_DIGITS + 1]; //Add one for '\0'
	int position = 0;
	int i;
	int leave_state = RUN_FINAL;
	char given[HS_MAX_INITIALS + 1];

	buf = hi_score_read();
	if (buf)
	{
		snprintf(score_buf, MAX_DIGITS + 1, "%0*d", MAX_DIGITS, score);
		for (i = 0; i < HS_MAX_DIGITS; i += MAX_DIGITS)
		{
			if (strncmp(score_buf, &buf[i], MAX_DIGITS) <= 0)
				position++;
		}
	}
	//show score even if not in hiscore
	_final_score_draw(score, position);

	//if desn't rank or we did not get the hiscore buf (sorry) bye bye
	if (!buf || position == HS_MAX_SCORES)
	{
		SDL_Delay(2000);
		if (buf)
			free(buf);
		return RUN_INTRO;
	}
	//Clear the queue
	while (SDL_PollEvent(&event))
		; //Empty

	SDL_EnableUNICODE(1);	//enable international keyboard overhead
	i = 0;
	memset(given, ' ', HS_MAX_INITIALS);
	while (leave_state == RUN_FINAL)
	{
		static SDL_Rect srect = {0, 0, 30, 30};
		SDL_Rect drect;
		char ch;

		//Get the players initials
		SDL_WaitEvent(&event);
		switch (event.type)
		{
			case SDL_QUIT:
				leave_state = RUN_QUIT;
				break;
			case SDL_KEYDOWN:
				ch = event.key.keysym.sym;
				if (ch >= 'a' && ch <= 'z')
				{
					drect.x = 300 + (40 * i);
					drect.y = 260;
					srect.x = (ch - 'a') * 30;
					SDL_BlitSurface(alpha_30_image, &srect, screen, &drect);
					SDL_UpdateRect(screen, drect.x, drect.y, drect.w, drect.h);
					given[i] = ch;
					++i;
				}
				else if (ch == SDLK_ESCAPE)
					leave_state = RUN_QUIT;
				else if (ch == SDLK_RETURN)
					leave_state = RUN_SCORE;
				else
				{
					given[i] = ' ';
					++i;
				}
				break;
			default:
				break;
		}
		if (i == HS_MAX_INITIALS+1)
		{
			leave_state = RUN_SCORE;
			break;
		}
		WaitFrame();
	}

	// Move lesser scores down in buf
	for (i = HS_MAX_SCORES - 1; i > position; --i)
	{
		int j;

		j = i * MAX_DIGITS;
		memcpy(&buf[j], &buf[j - MAX_DIGITS], MAX_DIGITS);
		j = HS_MAX_DIGITS + (i * HS_MAX_INITIALS);
		memcpy(&buf[j], &buf[j - HS_MAX_INITIALS], HS_MAX_INITIALS);
	}
	// Insert the current score
	memcpy(&buf[position * MAX_DIGITS], score_buf, MAX_DIGITS);
	memcpy(&buf[position * HS_MAX_INITIALS + HS_MAX_DIGITS], given, HS_MAX_INITIALS);
	// Save the file
	hi_score_write(buf);

	free(buf);
	return leave_state;
}

